==> seplib-6.3.5.tar.gz: seplib-6.3.5/seplib_base/lib/corelibs/superset/sect_local.c <==
#define SET_SDOC 1

#include<sitedef.h>
#include<superset.h>
#include<math.h>
#include<../include/superset_internal.h>

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_isect(char *tag, int iown,int *isect)
_XFUNCPROTOEND
#endif
{
sep_3d *info;

info = tag_info_sep3d(tag, INQUIRE);
if(info==SEPNULL)
 return(sepwarn(NOT_MET,"Not a valid tag %s \n",tag));
if(info->dff==SEPNULL) return(0);
if(info->dff->naxis_convert<iown || iown < 1)
  return(sepwarn(-1,"Invalid index %d axis %d length=%d \n", iown,info->dff->sect_axis,info->dff->naxis_convert));
return(info->dff->axis_convert[iown-1]);
}


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_iown(char *tag, int iown)
_XFUNCPROTOEND
#endif
{
sep_3d *info;

info = tag_info_sep3d(tag, INQUIRE);
if(info==SEPNULL)
 return(sepwarn(NOT_MET,"Not a valid tag %s \n",tag));
if(info->dff==SEPNULL) return(0);
if(info->dff->naxis_convert<iown || iown < 1)
  return(sepwarn(-1,"Invalid index %d axis %d length=%d \n", iown,info->dff->sect_axis,info->dff->naxis_convert));
if(info->dff->sect_thread[info->dff->axis_convert[iown-1]]==info->dff->impi) return(0);
return(1);

}


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEP3D_make_local_tag(sep_3d *info, int isect, char *section)
_XFUNCPROTOEND
#endif
{
int *nhs;
sep_3d *info_local;
int n;
float o,d;
char label[256],unit[256];

info_local = tag_info_sep3d(section, INQUIRE);  /* get info on this tag */
if(info_local != SEPNULL) sep3d_delete(section);

if(0!=sep3d_struct_init(info->dff->tag_sect[isect],section,"SCRATCH"))
  return(sepwarn(NOT_MET,"tag%s section=%d  trouble making local tag\n",info->name,isect));

 return(SUCCESS);
}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_grab_sect_nh(char *tag,  int *nsz)
_XFUNCPROTOEND
#endif
{
int n123,i,*grid;
sep_3d *info;
  
info = tag_info_sep3d(tag, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",tag));

for(i=1,n123=1; i<info->ndims; i++){
   n123=info->nwind[i]*n123;
}

if(info->file_format==GRID){
  grid=(int*) malloc(sizeof(int)*n123);

  if(0!=sep_get_grid_window(tag,&(info->ndims),&(info->n[1]),&
    info->nwind[1],&info->fwind[1],&info->jwind[1],grid)){
    free(grid);
    return(sepwarn(NOT_MET,"trouble grabbing grid window for tag=%s \n",tag));
  }
 
  for(i=0,*nsz=0;i < n123; i++)
    if(grid[i] >0) *nsz=*nsz+1;

   free(grid);
}
else *nsz=n123;
  return(SUCCESS);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEP3D_localize_window(sep_3d *info,sep_3d *info_local, int isect)
_XFUNCPROTOEND
#endif
{
int i;

for(i=1; i<= info->ndims; i++){
  if(0!=SEP3D_localize_axis(info,isect,i,info->nwind[i-1],info->fwind[i-1],info->jwind[i-1],
  &info_local->nwind[i-1],&info_local->fwind[i-1],&info_local->jwind[i-1]))
  return(sepwarn(NOT_MET,"tag%s section=%d trouble fixing window section\n",
   info->name,isect));
}
return(SUCCESS);
}


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEP3D_window_section_2(sep_3d *info,sep_3d *info_local, int isect)
_XFUNCPROTOEND
#endif
{
int i;

for(i=2; i<= info->ndims; i++){
  if(0!=SEP3D_localize_axis(info,isect,i,info->nwind[i-1],info->fwind[i-1],info->jwind[i-1],
  &info_local->nwind[i-1],&info_local->fwind[i-1],&info_local->jwind[i-1]))
  return(sepwarn(NOT_MET,"tag%s section=%d trouble fixing window section\n",
   info->name,isect));
}
return(SUCCESS);
}


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEP3D_localize_axis(sep_3d *info,int isect, int idimd, int nin, int fin, 
 int jin, int *nout, int *fout, int *jout)
_XFUNCPROTOEND
#endif
{
int idim;
int skip,beg,end,last,e,ifound;
int i,iglobal,ilocal,ifirst,ilast,ic,ival,n;
sep_3d *info_local;
sep_dff *info_dff;
int nremain,nt,f,j;
int ifirst_wind,ilast_wind,iax;
int ibeg,iend,iskip;
int fbeg,fskip;
int icon,nc;
float ffirst_small,flast_small;

info_dff=info->dff;

  idim=idimd-1;
  if(info_dff->sect_axis!=idimd){
    *nout=nin; *fout=fin; *jout=jin;
    return(0);
  }



iax=info_dff->sect_axis-1;
if(info_dff->master_data==1) ibeg=0;
else ibeg=1;

ifirst_wind=fin;
ilast_wind=fin+jin*(nin-1);

                                                                                                
if(info_dff->pattern==BLOCK){
  nt=0;nremain=info->n[iax];
  for(i=ibeg,f=0; i <= isect; i++){
    nt=ceil((double)nremain/((double)(info_dff->nsect-i)));
    nremain-=nt;
    f+=nt;
  }
  n=nt; f=f-nt; j=1;
}
else if(info_dff->pattern==SPREAD){
   j=info_dff->nsect-ibeg;
   f=isect-ibeg;
   n=info->n[iax]/(info_dff->nsect-ibeg);
   nremain=info->n[iax] - n*(info_dff->nsect-ibeg) ;
   if(nremain!=0){
     if(isect-ibeg < nremain) n+=1;
   }
}
fbeg=0; fskip=0;nc=0;
for(i=0,ival=fin; i < nin; i++,ival+=jin){
  if(ival%j==0){
    icon=(ival-f)/j;
    if(icon>=0 && icon < n){
      if(fbeg==0){ ibeg=icon; fbeg=1;}
      else{
        if(fskip==0) { fskip=1; iskip=icon-ibeg;}
      }
      iend=i;
      nc+=1;
    }
  }
 
}
if(fbeg==0){ *fout=0; *jout=1; *nout=0;return(0);}
*fout=ibeg;
if(fskip==0) *jout=1;
else  *jout=iskip;
*nout=nc;





/*


  info_local=tag_info_sep3d(info_dff->tag_sect[isect], INQUIRE);
 if(info_local == SEPNULL)
  return(sepwarn(INVALID_STRUC,"sect problem tag:%s \n",
    info_dff->tag_sect[isect]));
  if(info_dff->pattern==NODEF)
  return(sepwarn(NOT_MET,"pattern undefined for tag %s can't section \n",
  info_dff->name));



  if(info_dff->sect_axis!=idimd){
    *nout=nin; *fout=fin; *jout=jin;
  }
  else{
    beg=(int)rint((double)(info_local->o[idim]-
     info->o[idim])/ ((double)info->d[idim]));
    end=(int)rint((double)(info_local->o[idim]+info_local->d[idim]*
     (info_local->n[idim]-1)-info->o[idim])/((double)info->d[idim]));
    last=fin+jin*(nin-1);
    skip=rint((double)info_local->d[idim]/((double)info->d[idim]));

    ifirst=nin; ilast=-1;
    ic=0;
    for(i=0; i < nin; i++){
      iglobal=fin+jin*i;
      if((iglobal-beg)%skip==0 && iglobal >= beg && iglobal <= end){
        ilocal=(iglobal-beg)/skip;
        if(ilocal < ifirst) ifirst=ilocal;
        if(ilocal> ilast) ilast=ilocal;
        ic+=1;
      }
    }
    *fout=ifirst;
     if(ic>1) *jout=(ilast-ifirst)/(ic-1);
     else *jout=1;
    *nout=ic;
  }

*/

return(SUCCESS);
}

==> seplib-6.3.8.tar.gz: seplib-6.3.8/seplib_base/lib/corelibs/superset/sect_local.c <==
#define SET_SDOC 1

#include<sitedef.h>
#include<superset.h>
#include<math.h>
#include<superset_internal.h>

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_isect(char *tag, int iown,int *isect)
_XFUNCPROTOEND
#endif
{
sep_3d *info;

info = tag_info_sep3d(tag, INQUIRE);
if(info==SEPNULL)
 return(sepwarn(NOT_MET,"Not a valid tag %s \n",tag));
if(info->dff==SEPNULL) return(0);
if(info->dff->naxis_convert<iown || iown < 1)
  return(sepwarn(-1,"Invalid index %d axis %d length=%d \n", iown,info->dff->sect_axis,info->dff->naxis_convert));
return(info->dff->axis_convert[iown-1]);
}


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_iown(char *tag, int iown)
_XFUNCPROTOEND
#endif
{
sep_3d *info;

info = tag_info_sep3d(tag, INQUIRE);
if(info==SEPNULL)
 return(sepwarn(NOT_MET,"Not a valid tag %s \n",tag));
if(info->dff==SEPNULL) return(0);
if(info->dff->naxis_convert<iown || iown < 1)
  return(sepwarn(-1,"Invalid index %d axis %d length=%d \n", iown,info->dff->sect_axis,info->dff->naxis_convert));
if(info->dff->sect_thread[info->dff->axis_convert[iown-1]]==info->dff->impi) return(0);
return(1);

}


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEP3D_make_local_tag(sep_3d *info, int isect, char *section)
_XFUNCPROTOEND
#endif
{
int *nhs;
sep_3d *info_local;
int n;
float o,d;
char label[256],unit[256],temp_name[2049];

info_local = tag_info_sep3d(section, INQUIRE);  /* get info on this tag */
if(info_local != SEPNULL) sep3d_delete(section);

  SEPDFF_sect_name(info->dff,isect,temp_name);
if(0!=sep3d_struct_init(temp_name,section,"SCRATCH"))
  return(sepwarn(NOT_MET,"tag%s section=%d  trouble making local tag\n",info->name,isect));

 return(SUCCESS);
}
#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int sep3d_grab_sect_nh(char *tag,  int *nsz)
_XFUNCPROTOEND
#endif
{
int n123,i,*grid;
sep_3d *info;
  
info = tag_info_sep3d(tag, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  invalid struc\n",tag));

for(i=1,n123=1; i<info->ndims; i++){
   n123=info->nwind[i]*n123;
}

if(info->file_format==GRID){
  grid=(int*) malloc(sizeof(int)*n123);

  if(0!=sep_get_grid_window(tag,&(info->ndims),&(info->n[1]),&
    info->nwind[1],&info->fwind[1],&info->jwind[1],grid)){
    free(grid);
    return(sepwarn(NOT_MET,"trouble grabbing grid window for tag=%s \n",tag));
  }
 
  for(i=0,*nsz=0;i < n123; i++)
    if(grid[i] >0) *nsz=*nsz+1;

   free(grid);
}
else *nsz=n123;
  return(SUCCESS);
}

#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEP3D_localize_window(sep_3d *info,sep_3d *info_local, int isect)
_XFUNCPROTOEND
#endif
{
int i;

for(i=1; i<= info->ndims; i++){
  if(0!=SEP3D_localize_axis(info,isect,i,info->nwind[i-1],info->fwind[i-1],info->jwind[i-1],
  &info_local->nwind[i-1],&info_local->fwind[i-1],&info_local->jwind[i-1]))
  return(sepwarn(NOT_MET,"tag%s section=%d trouble fixing window section\n",
   info->name,isect));
}
return(SUCCESS);
}


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEP3D_window_section_2(sep_3d *info,sep_3d *info_local, int isect)
_XFUNCPROTOEND
#endif
{
int i;

for(i=2; i<= info->ndims; i++){
  if(0!=SEP3D_localize_axis(info,isect,i,info->nwind[i-1],info->fwind[i-1],info->jwind[i-1],
  &info_local->nwind[i-1],&info_local->fwind[i-1],&info_local->jwind[i-1]))
  return(sepwarn(NOT_MET,"tag%s section=%d trouble fixing window section\n",
   info->name,isect));
}
return(SUCCESS);
}


#if NeedFunctionPrototypes
_XFUNCPROTOBEGIN
int SEP3D_localize_axis(sep_3d *info,int isect, int idimd, int nin, int fin, 
 int jin, int *nout, int *fout, int *jout)
_XFUNCPROTOEND
#endif
{
int idim;
int skip,beg,end,last,e,ifound;
int i,iglobal,ilocal,ifirst,ilast,ic,ival,n;
sep_3d *info_local;
sep_dff *info_dff;
int nremain,nt,f,j;
int ifirst_wind,ilast_wind,iax;
int ibeg,iend,iskip;
int fbeg,fskip;
int icon,nc;
float ffirst_small,flast_small;

info_dff=info->dff;

  idim=idimd-1;
  if(info_dff->sect_axis!=idimd){
    *nout=nin; *fout=fin; *jout=jin;
    return(0);
  }



iax=info_dff->sect_axis-1;
if(info_dff->master_data==1) ibeg=0;
else ibeg=1;

ifirst_wind=fin;
ilast_wind=fin+jin*(nin-1);

                                                                                                
if(info_dff->pattern==BLOCK){
  nt=0;nremain=info->n[iax];
  for(i=ibeg,f=0; i <= isect; i++){
    nt=ceil((double)nremain/((double)(info_dff->nsect-i)));
    nremain-=nt;
    f+=nt;
  }
  n=nt; f=f-nt; j=1;
}
else if(info_dff->pattern==SPREAD){
   j=info_dff->nsect-ibeg;
   f=isect-ibeg;
   n=info->n[iax]/(info_dff->nsect-ibeg);
   nremain=info->n[iax] - n*(info_dff->nsect-ibeg) ;
   if(nremain!=0){
     if(isect-ibeg < nremain) n+=1;
   }
}
fbeg=0; fskip=0;nc=0;
for(i=0,ival=fin; i < nin; i++,ival+=jin){
  if(ival%j==0){
    icon=(ival-f)/j;
    if(icon>=0 && icon < n){
      if(fbeg==0){ ibeg=icon; fbeg=1;}
      else{
        if(fskip==0) { fskip=1; iskip=icon-ibeg;}
      }
      iend=i;
      nc+=1;
    }
  }
 
}
if(fbeg==0){ *fout=0; *jout=1; *nout=0;return(0);}
*fout=ibeg;
if(fskip==0) *jout=1;
else  *jout=iskip;
*nout=nc;





/*


  info_local=tag_info_section(info_dff,isect, INQUIRE);
 if(info_local == SEPNULL)
  return(sepwarn(INVALID_STRUC,"sect problem tag:%s \n",
    info_dff->tag_sect[isect]));
  if(info_dff->pattern==NODEF)
  return(sepwarn(NOT_MET,"pattern undefined for tag %s can't section \n",
  info_dff->name));



  if(info_dff->sect_axis!=idimd){
    *nout=nin; *fout=fin; *jout=jin;
  }
  else{
    beg=(int)rint((double)(info_local->o[idim]-
     info->o[idim])/ ((double)info->d[idim]));
    end=(int)rint((double)(info_local->o[idim]+info_local->d[idim]*
     (info_local->n[idim]-1)-info->o[idim])/((double)info->d[idim]));
    last=fin+jin*(nin-1);
    skip=rint((double)info_local->d[idim]/((double)info->d[idim]));

    ifirst=nin; ilast=-1;
    ic=0;
    for(i=0; i < nin; i++){
      iglobal=fin+jin*i;
      if((iglobal-beg)%skip==0 && iglobal >= beg && iglobal <= end){
        ilocal=(iglobal-beg)/skip;
        if(ilocal < ifirst) ifirst=ilocal;
        if(ilocal> ilast) ilast=ilocal;
        ic+=1;
      }
    }
    *fout=ifirst;
     if(ic>1) *jout=(ilast-ifirst)/(ic-1);
     else *jout=1;
    *nout=ic;
  }

*/

return(SUCCESS);
}
